from charset_normalizer.api import from_bytes
from charset_normalizer.constant import CHARDET_CORRESPONDENCE
from typing import Dict, Optional, Union


def detect(byte_str: bytes) -> Dict[str, Optional[Union[str, float]]]:
    """
    chardet legacy method
    Detect the encoding of the given byte string. It should be mostly backward-compatible.
    Encoding name will match Chardet own writing whenever possible. (Not on encoding name unsupported by it)
    This function is deprecated and should be used to migrate your project easily, consult the documentation for
    further information. Not planned for removal.

    :param byte_str:     The byte sequence to examine.
    """
    if not isinstance(byte_str, (bytearray, bytes)):
        raise TypeError('Expected object of type bytes or bytearray, got: '
                        '{0}'.format(type(byte_str)))

    if isinstance(byte_str, bytearray):
        byte_str = bytes(byte_str)

    r = from_bytes(byte_str).best()

    encoding = r.encoding if r is not None else None
    language = r.language if r is not None and r.language != 'Unknown' else ''
    confidence = 1. - r.chaos if r is not None else None

    # Note: CharsetNormalizer does not return 'UTF-8-SIG' as the sig get stripped in the detection/normalization process
    # but chardet does return 'utf-8-sig' and it is a valid codec name.
    if r is not None and encoding == 'utf_8' and r.bom:
        encoding += '_sig'

    return {
        'encoding': encoding if encoding not in CHARDET_CORRESPONDENCE else CHARDET_CORRESPONDENCE[encoding],
        'language': language,
        'confidence': confidence
    }



# coding: utf-8
import unittest
from glob import glob

from charset_normalizer.api import from_path
from os.path import basename


class TestFileCharsetNormalizer(unittest.TestCase):

    SHOULD_BE = {
        'sample.1.ar.srt': 'cp1256',
        'sample.1.fr.srt': 'cp1252',
        'sample.1.gr.srt': 'cp1253',
        'sample.1.he.srt': 'cp1255',
        'sample.1.hi.srt': 'ascii',
        'sample.1.ru.srt': 'cp1251',
        'sample.1.tu.srt': 'cp1254',
        'sample.2.ar.srt': 'cp1256',
        'sample.3.ar.srt': 'utf_8',
        'sample.4.ar.srt': 'cp1256',
        'sample.5.ar.srt': 'utf_8',
        'sample-chinese.txt': 'big5',
        'sample-greek.txt': 'cp1253',
        'sample-greek-2.txt': 'cp1253',
        'sample-hebrew.txt': 'utf_8',
        'sample-hebrew-2.txt': 'cp1255',
        'sample-hebrew-3.txt': 'cp1255',
        'sample-russian.txt': 'mac_cyrillic',
        'sample-russian-2.txt': 'utf_8',
        'sample-turkish.txt': 'cp1254',
        'sample-korean.txt': 'cp949',
        'sample-spanish.txt': 'utf_8',
        'sample-bulgarian.txt': 'utf_8',
        'sample-english.bom.txt': 'utf_8'
    }

    def test_file_input(self):
        for path_name in glob('./data/*.srt') + glob('./data/*.txt'):

            with self.subTest('test_file_input <{}>'.format(path_name)):

                matches = from_path(path_name)

                self.assertGreater(
                    len(matches),
                    0
                )

                r_ = matches.best().first()

                self.assertIsNotNone(
                    r_
                )

                if isinstance(TestFileCharsetNormalizer.SHOULD_BE[basename(path_name)], str):
                    self.assertEqual(
                        r_.encoding,
                        TestFileCharsetNormalizer.SHOULD_BE[basename(path_name)]
                    )
                else:
                    self.assertIn(
                        r_.encoding,
                        TestFileCharsetNormalizer.SHOULD_BE[basename(path_name)]
                    )


if __name__ == '__main__':
    unittest.main()



import unittest
from charset_normalizer.api import from_path
from glob import glob
from os.path import basename


class TestLanguageDetection(unittest.TestCase):
    SHOULD_BE = {
        'sample.1.ar.srt': 'Arabic',
        'sample.1.fr.srt': 'French',
        'sample.1.gr.srt': 'Greek',
        'sample.1.he.srt': 'Hebrew',
        'sample.1.hi.srt': 'English',
        'sample.1.ru.srt': 'Russian',
        'sample.1.tu.srt': 'Turkish',
        'sample.2.ar.srt': 'Arabic',
        'sample.3.ar.srt': 'Arabic',
        'sample.4.ar.srt': 'Arabic',
        'sample.5.ar.srt': 'Arabic',
        'sample-chinese.txt': 'Classical Chinese',
        'sample-greek.txt': 'Greek',
        'sample-greek-2.txt': 'Greek',
        'sample-hebrew.txt': 'English',
        'sample-hebrew-2.txt': 'Hebrew',
        'sample-hebrew-3.txt': 'Hebrew',
        'sample-russian.txt': 'Russian',
        'sample-russian-2.txt': 'Russian',
        'sample-turkish.txt': 'Turkish',
        'sample-korean.txt': 'Korean',
        'sample-spanish.txt': 'Spanish',
        'sample-bulgarian.txt': 'Bulgarian',
        'sample-english.bom.txt': 'English'
    }

    def test_language_detection(self):

        for path_name in glob('./data/*.srt') + glob('./data/*.txt'):
            with self.subTest(path_name+' WRITTEN IN '+TestLanguageDetection.SHOULD_BE[basename(path_name)]):
                r_ = from_path(path_name).best().first()

                self.assertEqual(
                    TestLanguageDetection.SHOULD_BE[basename(path_name)],
                    r_.language
                )


if __name__ == '__main__':
    unittest.main()



import unittest
from charset_normalizer.utils import any_specified_encoding


class TestDetectLegacy(unittest.TestCase):

    def test_specified_encoding_xml(self):

        p = any_specified_encoding(
            b'<?xml version="1.0" encoding="EUC-JP"?>'
        )

        self.assertEqual(
            "euc_jp",
            p
        )

    def test_specified_encoding_html(self):

        p = any_specified_encoding(
            b'<html><head><meta charset="utf-8"></head></html>'
        )

        self.assertEqual(
            "utf_8",
            p
        )

    def test_specified_but_unknown(self):
        p = any_specified_encoding(
            b'<html><head><meta charset="utf-57"></head></html>'
        )

        self.assertIsNone(
            p
        )



#!/bin/python
from glob import glob
from sys import argv
from typing import List
import argparse

from charset_normalizer import from_path
from charset_normalizer.utils import iana_name

from os import sep


def calc_equivalence(content: bytes, cp_a: str, cp_b: str):
    str_a = content.decode(cp_a)
    str_b = content.decode(cp_b)

    character_count = len(str_a)
    diff_character_count = 0

    for chr_a, chr_b in zip(str_a, str_b):
        if chr_a != chr_b:
            diff_character_count += 1

    return 1. - (diff_character_count / character_count)


def cli_coverage(arguments: List[str]):
    parser = argparse.ArgumentParser(
        description="Embedded detection success coverage script checker for Charset-Normalizer"
    )

    parser.add_argument('-p', '--with-preemptive', action="store_true", default=False, dest='preemptive',
                        help='Enable the preemptive scan behaviour during coverage check')
    parser.add_argument('-c', '--coverage', action="store", default=90, type=int, dest='coverage',
                        help="Define the minimum acceptable coverage to succeed")

    args = parser.parse_args(arguments)

    success_count = 0
    total_count = 0

    for tbt_path in glob("./char-dataset/**/*.*"):

        expected_encoding = tbt_path.split(sep)[-2]
        total_count += 1

        results = from_path(
            tbt_path,
            preemptive_behaviour=args.preemptive
        )

        if len(results) == 0:
            print("⚡⚡ '{}' (nothing)".format(tbt_path))
            continue

        result = results.best()

        if expected_encoding in result.could_be_from_charset or iana_name(expected_encoding) in result.could_be_from_charset:
            print("✅✅ '{}'".format(tbt_path))
            success_count += 1
            continue

        calc_eq = calc_equivalence(result.raw, expected_encoding, result.encoding)

        if calc_eq >= 0.98:
            success_count += 1
            print("️✅ ️'{}' (got '{}' but equivalence {} %)".format(tbt_path, result.encoding, round(calc_eq * 100., 3)))
            continue

        print("⚡ '{}' (got '{}')".format(tbt_path, result.encoding))

    success_ratio = round(success_count / total_count, 2) * 100.

    print("Total EST coverage = {} % ({} / {} files)".format(success_ratio, success_count, total_count))

    return 0 if success_ratio >= args.coverage else 1


if __name__ == "__main__":
    exit(
        cli_coverage(
            argv[1:]
        )
    )



import unittest

from charset_normalizer.api import from_bytes
from charset_normalizer.constant import TOO_BIG_SEQUENCE


class TestBytes(unittest.TestCase):

    def test_too_short_none(self):
        self.assertIsNotNone(
            from_bytes(b'\xfe\xff').best().first()
        )

    def test_empty_bytes(self):
        r = from_bytes(b'').best().first()

        self.assertIsNotNone(
            r
        )

        self.assertEqual(
            'utf_8',
            r.encoding
        )

        self.assertEqual(
            0,
            len(r.alphabets)
        )

    def test_ensure_ascii_fallback(self):
        payload = b"AbAdZ pOoooOlDl mmlDoDkA lldDkeEkddA mpAlkDF"
        r = from_bytes(payload).best()

        self.assertIsNotNone(
            r,
            msg="Fallback ASCII detection has failed. You clearly have tempered with it. Testing with {}".format(payload)
        )

        self.assertEqual(
            r.encoding,
            "ascii",
            msg="Fallback ASCII miss-detection. You clearly have tempered with it. Testing with {}".format(payload)
        )

    def test_ensure_u8_fallback(self):
        payload = '\u020d\x1b'.encode('utf-8')
        r = from_bytes(payload).best()

        self.assertIsNotNone(
            r,
            msg="Fallback UTF-8 detection has failed. You clearly have tempered with it. Testing with {}".format(payload)
        )

        self.assertEqual(
            r.encoding,
            "utf_8",
            msg="Fallback UTF-8 miss-detection. You clearly have tempered with it. Testing with {}".format(payload)
        )

    def test_alphabets_property_undefined_range(self):
        payload = b'\xef\xbb\xbf\xf0\x9f\xa9\xb3'

        r = from_bytes(payload)

        self.assertEqual(
            r.best().encoding,
            "utf_8"
        )

        self.assertEqual(
            r.best().alphabets,
            []
        )

    def test_ensure_ascii(self):

        for payload in [
            b"g4UsPJdfzNkGW2jwmKDGDilKGKYtpF2X.mx3MaTWL1tL7CNn5U7DeCcodKX7S3lwwJPKNjBT8etY",
            b'{"token": "g4UsPJdfzNkGW2jwmKDGDilKGKYtpF2X.mx3MaTWL1tL7CNn5U7DeCcodKX7S3lwwJPKNjBT8etY"}',
            b"81f4ab054b39cb0e12701e734077d84264308f5fc79494fc5f159fa2ebc07b73c8cc0e98e009664a20986706f90146e8eefcb929ce1f74a8eab21369fdc70198"
        ]:
            r = from_bytes(payload).best()

            self.assertIsNotNone(
                r,
                msg="Simple ASCII detection has failed. You clearly have tempered with it. Testing with {}".format(payload)
            )

            self.assertEqual(
                r.encoding,
                "ascii",
                msg="Simple ASCII miss-detection. You clearly have tempered with it. Testing with {}".format(payload)
            )

    def test_empty_str_with_sig_gb18030(self):
        r = from_bytes('\uFEFF'.encode('gb18030')).best()

        self.assertIsNotNone(r)
        self.assertEqual(
            "",
            str(r)
        )
        self.assertEqual(
            "gb18030",
            r.encoding
        )
        self.assertEqual(
            4,
            len(r.raw)
        )

    def test_empty_str_with_sig_utf8(self):
        r = from_bytes(b'\xef\xbb\xbf').best()

        self.assertIsNotNone(r)
        self.assertEqual(
            "",
            str(r)
        )
        self.assertEqual(
            "utf_8",
            r.encoding
        )
        self.assertEqual(
            3,
            len(r.raw)
        )

    def test_empty_str_with_large_sig_utf8(self):
        r = from_bytes(b'\xef\xbb\xbf' + (b'0' * TOO_BIG_SEQUENCE)).best()

        self.assertIsNotNone(r)
        self.assertEqual(
            '0' * TOO_BIG_SEQUENCE,
            str(r)
        )
        self.assertEqual(
            "utf_8",
            r.encoding
        )
        self.assertEqual(
            TOO_BIG_SEQUENCE + 3,
            len(r.raw)
        )

    def test_on_empty_json(self):

        with self.subTest("Detecting empty JSON as ASCII"):
            results = from_bytes(b"{}").best()
            self.assertIsNotNone(
                results.best()
            )
            self.assertEqual(
                results.best().encoding,
                "ascii"
            )

    def test_bom_detection(self):
        with self.subTest('GB18030 UNAVAILABLE SIG'):
            self.assertFalse(
                from_bytes(
                    '我没有埋怨，磋砣的只是一些时间。。今觀俗士之論也，以族舉德，以位命賢，茲可謂得論之一體矣，而未獲至論之淑真也。'.encode('gb18030')
                ).best().first().byte_order_mark
            )

        with self.subTest('GB18030 AVAILABLE SIG'):
            self.assertTrue(
                from_bytes(
                    (u'\uFEFF' + '我没有埋怨，磋砣的只是一些时间。').encode('gb18030')
                ).best().first().byte_order_mark
            )

        with self.subTest('UTF-32 AVAILABLE BOM'):
            self.assertTrue(
                from_bytes(
                   '我没有埋怨，磋砣的只是一些时间。'.encode('utf_32')
                ).best().first().byte_order_mark
            )

        with self.subTest('UTF-8 AVAILABLE BOM'):
            self.assertTrue(
                from_bytes(
                    b'\xef\xbb\xbf' + '我没有埋怨，磋砣的只是一些时间。'.encode('utf_8')
                ).best().first().byte_order_mark
            )

    def test_encode_decode(self):

        with self.subTest('Encode & Detect UTF-8 WITHOUT SIG SMALL CONTENT'):
            self.assertEqual(
                from_bytes(
                    'h\xe9llo world!\n'.encode('utf_8')
                ).best().first().encoding,
                'utf_8'
            )

        with self.subTest('Encode & Detect GB18030 WITHOUT SIG'):
            self.assertEqual(
                from_bytes(
                    '我没有埋怨，磋砣的只是一些时间。夫令譽從我興，而二命自天降之。'.encode('gb18030')
                ).best().first().encoding,
                'gb18030'
            )

        with self.subTest('Encode & Detect GB18030 WITH SIG (CJK)'):
            self.assertEqual(
                from_bytes(
                    (u'\uFEFF' + '我没有埋怨，磋砣的只是一些时间。').encode('gb18030')
                ).best().first().encoding,
                'gb18030'
            )

        with self.subTest('Encode & Detect UTF-8 WITHOUT SIG (CJK)'):
            self.assertEqual(
                from_bytes(
                    '我没有埋怨，磋砣的只是一些时间。'.encode('utf_8')
                ).best().first().encoding,
                'utf_8'
            )

        with self.subTest('Encode & Detect UTF-32 WITH BOM (CJK)'):
            self.assertEqual(
                from_bytes(
                    '我没有埋怨，磋砣的只是一些时间。'.encode('utf_32')
                ).best().first().encoding,
                'utf_32'
            )

        with self.subTest('Encode & Detect UTF-8 WITH SIG (CJK)'):
            self.assertEqual(
                from_bytes(
                   b'\xef\xbb\xbf' + '我没有埋怨，磋砣的只是一些时间。'.encode('utf_8')
                ).best().first().encoding,
                'utf_8'
            )

        with self.subTest('Encode & Detect UTF-8 WITHOUT SIG (CYRILLIC)'):
            self.assertEqual(
                from_bytes(
                    'Bсеки човек има право на образование. Oбразованието трябва да бъде безплатно, '
                    'поне що се отнася до началното и основното образование.'.encode('utf_8')
                ).best().first().encoding,
                'utf_8'
            )

        with self.subTest('Encode & Detect UTF-8 WITHOUT SIG (CYRILLIC)'):
            self.assertEqual(
                from_bytes(
                    'Bсеки човек има право на образование.'.encode(
                        'utf_8')
                ).best().first().encoding,
                'utf_8'
            )



from requests import get, __version__
from typing import List
from charset_normalizer import detect, __version__ as __version_cn__

if __name__ == "__main__":

    print(f"requests {__version__}")
    print(f"charset_normalizer {__version_cn__}")

    files: List[str] = get("http://127.0.0.1:8080/").json()

    print("## Testing with actual files")

    for file in files:
        r = get(
            "http://127.0.0.1:8080/" + file
        )

        if r.ok is False:
            print(f"Unable to retrieve '{file}' | HTTP/{r.status_code}")
            exit(1)

        expected_encoding = detect(r.content)["encoding"]

        if expected_encoding != r.apparent_encoding:
            print(f"Integration test failed | File '{file}' | Expected '{expected_encoding}' got '{r.apparent_encoding}'")
            exit(1)

        print(f"✅✅ '{file}' OK")

    print("## Testing with edge cases")

    # Should NOT crash
    get("http://127.0.0.1:8080/edge/empty/json").json()

    print("✅✅ Empty JSON OK")

    if get("http://127.0.0.1:8080/edge/empty/plain").apparent_encoding != "utf-8":
        print("Empty payload SHOULD not return apparent_encoding != UTF-8")
        exit(1)

    print("✅✅ Empty Plain Text OK")

    r = get("http://127.0.0.1:8080/edge/gb18030/json")

    if r.apparent_encoding != "GB18030":
        print("JSON Basic Detection FAILURE (/edge/gb18030/json)")
        exit(1)

    r.json()

    print("✅✅ GB18030 JSON Encoded OK")

    print("Integration tests passed!")



#!/bin/python
from glob import glob
from sys import argv
from typing import List
import argparse

from charset_normalizer import detect as tbt_detect
from chardet import detect as chardet_detect

from charset_normalizer.utils import iana_name


def calc_equivalence(content: bytes, cp_a: str, cp_b: str):
    try:
        str_a = content.decode(cp_a)
        str_b = content.decode(cp_b)
    except UnicodeDecodeError:
        return 0.

    character_count = len(str_a)
    diff_character_count = 0

    for chr_a, chr_b in zip(str_a, str_b):
        if chr_a != chr_b:
            diff_character_count += 1

    return 1. - (diff_character_count / character_count)


def cli_bc(arguments: List[str]):
    parser = argparse.ArgumentParser(
        description="BC script checker for Charset-Normalizer with Chardet"
    )

    parser.add_argument('-c', '--coverage', action="store", default=85, type=int, dest='coverage',
                        help="Define the minimum acceptable coverage to succeed")

    args = parser.parse_args(arguments)

    success_count = 0
    total_count = 0

    for tbt_path in glob("./char-dataset/**/*.*"):
        total_count += 1

        with open(tbt_path, "rb") as fp:
            content = fp.read()

        chardet_result = chardet_detect(content)
        chardet_encoding = chardet_result['encoding']

        charset_normalizer_result = tbt_detect(content)
        charset_normalizer_encoding = charset_normalizer_result['encoding']

        if [chardet_encoding, charset_normalizer_encoding].count(None) == 1:
            print("⚡⚡ '{}' (BC-Break) New('{}') vs Legacy('{}')".format(tbt_path, charset_normalizer_encoding, chardet_encoding))
            continue

        if charset_normalizer_encoding == chardet_encoding:
            success_count += 1
            print("✅✅ '{}' (BC)".format(tbt_path))
            continue

        if (chardet_encoding is None and charset_normalizer_encoding is None) or (iana_name(chardet_encoding) == iana_name(charset_normalizer_encoding)):
            success_count += 1
            print("✅✅ '{}' (BC)".format(tbt_path))
            continue

        calc_eq = calc_equivalence(content, chardet_encoding, charset_normalizer_encoding)

        if calc_eq >= 0.98:
            success_count += 1
            print("️✅ ️'{}' (got '{}' but eq {} WITH {} %)".format(tbt_path, charset_normalizer_encoding, chardet_encoding, round(calc_eq * 100., 3)))
            continue

        print("⚡⚡ '{}' (BC-Break) New('{}') vs Legacy('{}')".format(tbt_path, charset_normalizer_encoding, chardet_encoding))

    success_ratio = round(success_count / total_count, 2) * 100.

    print("Total EST BC = {} % ({} / {} files)".format(success_ratio, success_count, total_count))

    return 0 if success_ratio >= args.coverage else 1


if __name__ == "__main__":
    exit(
        cli_bc(
            argv[1:]
        )
    )



import unittest
from charset_normalizer.legacy import detect


class TestDetectLegacy(unittest.TestCase):

    def test_detect_dict_keys(self):

        r = detect(
            (u'\uFEFF' + '我没有埋怨，磋砣的只是一些时间。').encode('gb18030')
        )

        with self.subTest('encoding key present'):
            self.assertIn(
                'encoding',
                r.keys()
            )

        with self.subTest('language key present'):
            self.assertIn(
                'language',
                r.keys()
            )

        with self.subTest('confidence key present'):
            self.assertIn(
                'confidence',
                r.keys()
            )

    def test_detect_dict_value_type(self):

        r = detect(
            '我没有埋怨，磋砣的只是一些时间。'.encode('utf_8')
        )

        with self.subTest('encoding instance of str'):
            self.assertIsInstance(
                r['encoding'],
                str
            )

        with self.subTest('language instance of str'):
            self.assertIsInstance(
                r['language'],
                str
            )

        with self.subTest('confidence instance of float'):
            self.assertIsInstance(
                r['confidence'],
                float
            )

    def test_detect_dict_value(self):
        r = detect(
            '我没有埋怨，磋砣的只是一些时间。'.encode('utf_32')
        )

        with self.subTest('encoding is equal to utf_32'):
            self.assertEqual(
                r['encoding'],
                'UTF-32'
            )

    def test_utf8_sig_not_striped(self):
        r = detect(
            "Hello World".encode('utf-8-sig')
        )

        with self.subTest("Verify that UTF-8-SIG is returned when using legacy detect"):
            self.assertEqual(
                r['encoding'],
                "UTF-8-SIG"
            )



import unittest
from charset_normalizer.cli.normalizer import cli_detect, query_yes_no
from unittest.mock import patch
from os.path import exists
from os import remove


class TestCommandLineInterface(unittest.TestCase):

    @patch('builtins.input', lambda *args: 'y')
    def test_simple_yes_input(self):
        self.assertTrue(
            query_yes_no('Are u willing to chill a little bit ?')
        )

    @patch('builtins.input', lambda *args: 'N')
    def test_simple_no_input(self):
        self.assertFalse(
            query_yes_no('Are u willing to chill a little bit ?')
        )

    def test_single_file(self):

        self.assertEqual(
            0,
            cli_detect(
                ['./data/sample.1.ar.srt']
            )
        )

    def test_single_file_normalize(self):
        self.assertEqual(
            0,
            cli_detect(
                ['./data/sample.1.ar.srt', '--normalize']
            )
        )

        self.assertTrue(
            exists('./data/sample.1.ar.cp1256.srt')
        )

        try:
            remove('./data/sample.1.ar.cp1256.srt')
        except:
            pass

    def test_single_verbose_file(self):
        self.assertEqual(
            0,
            cli_detect(
                ['./data/sample.1.ar.srt', '--verbose']
            )
        )

    def test_multiple_file(self):
        self.assertEqual(
            0,
            cli_detect(
                [
                    './data/sample.1.ar.srt',
                    './data/sample.1.he.srt',
                    './data/sample-chinese.txt'
                ]
            )
        )

    def test_non_existent_file(self):

        with self.assertRaises(SystemExit) as cm:
            cli_detect(
                ['./data/not_found_data.txt']
            )

        self.assertEqual(cm.exception.code, 2)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
import unittest
from charset_normalizer.md import mess_ratio


class TestProbeChaos(unittest.TestCase):

    def test_not_gibberish(self):

        with self.subTest('Chinese Not Gibberish'):
            self.assertLessEqual(
                mess_ratio(
                    '典肇乎庚辰年十二月廿一，及己丑年二月十九，收各方語言二百五十，合逾七百萬目；二十大卷佔八成，單英文卷亦過二百萬。悉文乃天下有志共筆而成；有意助之，幾網路、隨纂作，大典茁焉。'),
                0.
            )

        with self.subTest('Arabic Not Gibberish'):
            self.assertEqual(
                mess_ratio('العقلية , التنويم المغناطيسي و / أو الاقتراح'),
                0.
            )

        with self.subTest('Arabic Styled Not Gibberish'):
            self.assertEqual(
                mess_ratio("RadoZ تـــعــــديــل الـــتــــوقــيــــت مـــن قــبــل"),
                0.
            )

    def test_subtle_gibberish(self):

        self.assertLessEqual(
            mess_ratio("Cehennemin Sava■þ²s²'da kim?"),
            0.5
        )

        self.assertGreaterEqual(
            mess_ratio("Cehennemin Sava■þ²s²'da kim?"),
            0.
        )

        self.assertGreater(
            mess_ratio("´Á¥½³ø§i --  ±i®Ìºû, ³¯·Ø©v"),
            0.5
        )

        self.assertGreater(
            mess_ratio("ïstanbul, T■rkiye'nin en kalabal»k, iktisadi ve k■lt■rel aÓ»dan en —nemli"),
            0.
        )

        self.assertLessEqual(
            mess_ratio("ïstanbul, T■rkiye'nin en kalabal»k, iktisadi ve k■lt■rel aÓ»dan en —nemli"),
            0.5
        )

        self.assertLessEqual(
            mess_ratio("<i>Parce que Óa, c'est la vÕritable histoire de la rencontre avec votre Tante Robin.</i>"),
            0.5
        )

    def test_complete_gibberish(self):
        self.assertTrue(
            mess_ratio("""ØĢØŠØģØ§ØĶŲ ŲŲ ØĢŲ Ø§ŲŲØ§Øģ ŲŲŲ ŲØ§ ØģŲŲŲØŠØģØ§ØĶŲŲŲØ ØŊØđŲØ§ ŲØģŲØđ ØđŲ (ŲØąŲØŊŲ) ŲØ§ŲØŪØ§ØŠŲ""") >= 0.1,
        )

        self.assertTrue(
            mess_ratio("""ÇáÚŞáíÉ , ÇáÊäæíã ÇáãÛäÇØíÓí æ / Ãæ ÇáÇŞÊÑÇÍ""") >= 0.1,
        )

    def test_part_gibberish(self):

        self.assertGreater(
            mess_ratio(
                """hishamkoc@yahoo.com ุชุฑุฌูููุฉ ููุดูููุงู ุงููููููููุงูRadoZ ุชูููุนููููุฏูููู ุงููููุชูููููููููููููุช ููููู ูููุจููู""", 0.5),
            0.3
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
import unittest
from charset_normalizer.api import from_path


class TestProbeCoherence(unittest.TestCase):
    def test_obvious_coherence_gap(self):

        arabic_coherence = from_path('./data/sample.1.ar.srt').best().first().coherence

        self.assertGreater(
            arabic_coherence,
            0.7
        )


if __name__ == '__main__':
    unittest.main()



from flask import Flask, jsonify, send_from_directory
from glob import glob

app = Flask(__name__)


@app.route('/raw/<path:path>')
def read_file(path):
    return send_from_directory('../char-dataset', path, as_attachment=True), 200, {"Content-Type": "text/plain"}


@app.route("/")
def read_targets():
    return jsonify(
        [
            el.replace("./char-dataset", "/raw").replace("\\", "/") for el in glob("./char-dataset/**/*")
        ]
    )


@app.route("/edge/empty/plain")
def read_empty_response_plain():
    return b"", 200, {"Content-Type": "text/plain"}


@app.route("/edge/empty/json")
def read_empty_response_json():
    return b"{}", 200, {"Content-Type": "application/json"}


@app.route("/edge/gb18030/json")
def read_gb18030_response_json():
    return '{"abc": "我没有埋怨，磋砣的只是一些时间。。今觀俗士之論也，以族舉德，以位命賢，茲可謂得論之一體矣，而未獲至論之淑真也。"}'.encode("gb18030"), 200, {"Content-Type": "application/json"}


if __name__ == "__main__":
    app.run(host="127.0.0.1", port=8080)



# coding: utf-8
import unittest
import charset_normalizer.utils as unicode_utils


class TestUnicodeHelper(unittest.TestCase):

    def test_should_be_accented(self):

        self.assertTrue(
            unicode_utils.is_accentuated('é')
        )

        self.assertTrue(
            unicode_utils.is_accentuated('è')
        )

        self.assertTrue(
            unicode_utils.is_accentuated('è')
        )

        self.assertTrue(
            unicode_utils.is_accentuated('à')
        )

        self.assertTrue(
            unicode_utils.is_accentuated('À')
        )

        self.assertTrue(
            unicode_utils.is_accentuated('Ù')
        )

        self.assertTrue(
            unicode_utils.is_accentuated('ç')
        )


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
from re import search

from setuptools import find_packages, setup


def get_version():
    with open('charset_normalizer/version.py') as version_file:
        return search(r"""__version__\s+=\s+(['"])(?P<version>.+?)\1""",
                      version_file.read()).group('version')


# Package meta-data.
NAME = 'charset-normalizer'
DESCRIPTION = 'The Real First Universal Charset Detector. Open, modern and actively maintained alternative to Chardet.'
URL = 'https://github.com/ousret/charset_normalizer'
EMAIL = 'ahmed.tahri@cloudnursery.dev'
AUTHOR = 'Ahmed TAHRI @Ousret'
REQUIRES_PYTHON = '>=3.5.0'
VERSION = get_version()

REQUIRED = []

EXTRAS = {
    'unicode_backport': ['unicodedata2']
}

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description.replace(':heavy_check_mark:', '✅'),
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    keywords=['encoding', 'i18n', 'txt', 'text', 'charset', 'charset-detector', 'normalization', 'unicode', 'chardet'],
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license='MIT',
    entry_points={
        'console_scripts':
            [
                'normalizer = charset_normalizer.cli.normalizer:cli_detect'
            ]
    },
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Text Processing :: Linguistic',
        'Topic :: Utilities',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)



#!/bin/python
from glob import glob
from time import time_ns
import argparse
from sys import argv

from charset_normalizer import detect
from chardet import detect as chardet_detect

from statistics import mean
from math import ceil


def calc_percentile(data, percentile):
    n = len(data)
    p = n * percentile / 100
    sorted_data = sorted(data)

    return sorted_data[int(p)] if p.is_integer() else sorted_data[int(ceil(p)) - 1]


def performance_compare(arguments):
    parser = argparse.ArgumentParser(
        description="Performance CI/CD check for Charset-Normalizer"
    )

    parser.add_argument('-s', '--size-increase', action="store", default=1, type=int, dest='size_coeff',
                        help="Apply artificial size increase to challenge the detection mechanism further")

    args = parser.parse_args(arguments)

    chardet_results = []
    charset_normalizer_results = []

    for tbt_path in glob("./char-dataset/**/*.*"):
        print(tbt_path)

        # Read Bin file
        with open(tbt_path, "rb") as fp:
            content = fp.read() * args.size_coeff

        before = time_ns()
        chardet_detect(content)
        chardet_results.append(
            round((time_ns() - before) / 1000000000, 5)
        )
        print("  --> Chardet: " + str(chardet_results[-1]) + "s")
        before = time_ns()
        detect(content)
        charset_normalizer_results.append(
            round((time_ns() - before) / 1000000000, 5)
        )
        print("  --> Charset-Normalizer: " + str(charset_normalizer_results[-1]) + "s")

    chardet_avg_delay = mean(chardet_results)
    chardet_99p = calc_percentile(chardet_results, 99)
    chardet_95p = calc_percentile(chardet_results, 95)
    chardet_50p = calc_percentile(chardet_results, 50)

    charset_normalizer_avg_delay = mean(charset_normalizer_results)
    charset_normalizer_99p = calc_percentile(charset_normalizer_results, 99)
    charset_normalizer_95p = calc_percentile(charset_normalizer_results, 95)
    charset_normalizer_50p = calc_percentile(charset_normalizer_results, 50)

    print("")

    print("------------------------------")
    print("--> Chardet Conclusions")
    print("   --> Avg: " + str(chardet_avg_delay) + "s")
    print("   --> 99th: " + str(chardet_99p) + "s")
    print("   --> 95th: " + str(chardet_95p) + "s")
    print("   --> 50th: " + str(chardet_50p) + "s")

    print("------------------------------")
    print("--> Charset-Normalizer Conclusions")
    print("   --> Avg: " + str(charset_normalizer_avg_delay) + "s")
    print("   --> 99th: " + str(charset_normalizer_99p) + "s")
    print("   --> 95th: " + str(charset_normalizer_95p) + "s")
    print("   --> 50th: " + str(charset_normalizer_50p) + "s")

    return 0 if chardet_avg_delay > charset_normalizer_avg_delay and chardet_99p > charset_normalizer_99p else 1


if __name__ == "__main__":
    exit(
        performance_compare(
            argv[1:]
        )
    )
